package aexp.btdetect;

import android.app.Service;
import android.content.Context;
import android.os.Bundle;
import android.os.DeadObjectException;
import android.os.IBinder;
import android.os.Handler;
import android.server.BluetoothService;
import android.util.Log;
import org.bluez.Manager;
import org.bluez.Adapter;
import org.bluez.IBluetoothService;
import org.bluez.BluetoothServiceNative;

class BtSimulatorTask implements Runnable {
  private static final String LOG_TAG = "BTSIMULATORTASK";
  String[] task;
  IBluetoothService is;
  BluetoothService bs;

  public BtSimulatorTask( IBluetoothService is,
			BluetoothService bs,
			String[] task ) {
	this.is = is;
    this.task = task;
	this.bs = bs;
  }

  public void run() {
    String typeChar = task[0];
    String addressString = task[1];
	if( typeChar.equals( "S" ) ) {
	  bs.notifyDiscoveryStarted();
	  Log.i( LOG_TAG, "notifyDiscoveryStarted()" );
	} else
	if( typeChar.equals( "E" ) ) {
	  bs.notifyDiscoveryCompleted();
	  try {
	    boolean bluetoothStopped = is.stopBluetooth( true );
	    if( !bluetoothStopped ) {
		  Log.e( LOG_TAG, "BT cannot be stopped" );
	    }
	  } catch( DeadObjectException ex ) {
		Log.e( LOG_TAG, "DeadObjectException" );
	  }
	  Log.i( LOG_TAG, "notifyDiscoveryCompleted()" );
	} else
	if( typeChar.equals( "A" ) ) {
      int btClass = 0;
      int rssi = 0;
      try {
	    btClass = Integer.parseInt( task[2] );
	    rssi = Integer.parseInt( task[3] );
      } catch( NumberFormatException ex ) {
	    Log.e( LOG_TAG,
		"Error parsing btClass,rssi elements in line: "+
			task[2]+","+task[3],ex );
	  }
	  bs.notifyRemoteDeviceFound( addressString, 
					btClass, 
					(short)rssi );
	  Log.i( LOG_TAG,
			"notifyRemoteDeviceFound("+addressString+
					","+
					btClass+
					","+
					rssi+
					")" );
	} else
	if( typeChar.equals( "N" ) ) {
	  String nameString = task[2];	
	  bs.notifyRemoteNameUpdated( addressString,nameString );
	  Log.i( LOG_TAG,
			"notifyRemoteNameUpdated("+addressString+
				","+
				nameString+")" );
	}
  }  
}

public class BtDetectSimulatorService extends Service {
  private static final String LOG_TAG = "BTDETECTSIMULATORSERVICE";
  final static String[][] testSeq = {
	{ "S", null,null,null,"1" },	/* 0 */
    { "A", "00:CD:FE:75:12:01", "2","100", "5" }, /* 1 */
	{ "N", "00:CD:FE:75:12:01", "Phone #1", "0","3" },	/* 2 */
	{ "A", "91:00:60:72:7A:15", "1","76","4" },		/* 3 */
	{ "N", "91:00:60:72:7A:15", "My headset","0","2" }, /* 4 */
	{ "E", null, null, null,"1" }	/* 5 */
  };
  private Handler serviceHandler = new Handler();

  @Override
  protected void onStart(int startId, Bundle arguments) {
	super.onStart( startId, arguments );
    Log.d( LOG_TAG, "onStart" );
	IBluetoothService service = BluetoothServiceNative.getBluetoothService();
	if( service == null ) {
	  Log.e( LOG_TAG, "Bluetooth service not registered" );
	  stopSelf();
	  return;
	}
    try {
	  Manager manager = service.getManager();
	  if( manager == null ) {
		Log.e( LOG_TAG,"manager==null" );
		stopSelf();
	  }
	  boolean bluetoothStarted = service.startBluetooth();
	  if( !bluetoothStarted ) {
	    Log.e( LOG_TAG, "BT cannot be started" );
		stopSelf();
	  }
/* Now we simulate the discovery process by sending out the following
   broadcasts.
   notifyDiscoveryStarted org.bluez.intent.action.DISCOVERY_STARTED
   notifyRemoteDeviceFound org.bluez.intent.action.REMOTE_DEVICE_FOUND
     address String BT address
     class Integer BT class
     rssi Short Received Signal Strength
   notifyRemoteNameUpdated org.bluez.intent.action.REMOTE_NAME_UPDATED
	 address String BT address
	 name String Device name
    notifyDiscoveryCompleted org.bluez.intent.action.DISCOVERY_COMPLETED
*/
/*
	android.server.BluetoothService is never instantiated like
	this as it is a system service. IBluetoothService does not
	provide access yet to the notify* methods we need.
*/
	  BluetoothService bs = new BluetoothService( (Context)this );
	  long delay = 0L;
	  for( int i = 0 ; i < testSeq.length ; ++i ) {
	    try {
		  int timeToWait = Integer.parseInt( testSeq[i][4] );
		  delay += (long)timeToWait*1000L;
		  BtSimulatorTask bst = new BtSimulatorTask( service, bs, testSeq[i] );
		  serviceHandler.postDelayed( bst,delay );
		  Log.d( LOG_TAG, "simulator task posted at "+
					delay+" msec" );
        } catch( NumberFormatException ex ) {
		  Log.e( LOG_TAG, "Unable to parse delay value: "+
				testSeq[i][4] );
        }
      }
    } catch( DeadObjectException ex ) {
	  Log.e( LOG_TAG,"DeadObjectException" );
	}
	stopSelf();
  }

  @Override
  protected void onDestroy() {
	super.onDestroy();
	Log.d( LOG_TAG, "onDestroy" );
  }

  @Override
  public IBinder getBinder() {
	return null;
  }
}
